\name{dcoex}
\alias{dcoex}
\title{ dcoex (differentially coexpressed genes) }
\description{
	Function to find groups of differentially co-expressed genes. Interface to C-code implementing an
	algorithm introduced by (Kostka et al.).
}

\usage{
dcoex(sample1, sample2, startmatrix = NULL, alpha = NULL, incdata = FALSE)
}

\arguments{
  \item{sample1}{	matrix or exprSet, containing of gene expression values of the 'control' group. 
			If matrix, rows are genes and columns are samples. 
                }
  \item{sample2}{ 	matrix or exprSet, containing of gene expression values of the 'disease' group. 
			If matrix, rows are genes and columns are samples.
                  }
  \item{startmatrix}{	if a (binary) matrix is passed, it is  coding for starting points of the search 
			algorithm. It has to have the same number of rows as 'sample1' and 'sample2'. Each
			column represents a starting point. A one codes for a gene included in the
			starting group, a zero for an excluded gene. For each column a search is performed.
	                If it is  an (integer) number, so many random starting points are generated and 
			for each one a search is performed. If \code{startmatrix}="kmeans" dcoex.kmeans 
			generates a startmatrix by using kmeans-clustering.
                    }
  \item{alpha}{
                      	numeric, tuning parameter in [0,1]. Larger values of alpha yield smaller groups 
			of genes and vice versa.
              }
  \item{incdata}{
                      	if you want to have datasets bigger than altogether 1000000 values
                      	in the dcoex-output you have to set incadata = TRUE\cr
                }

}

\value{			???????
			list, containing the following components: \code{nsample1.cols} is the
			number of samples in the control group, \code{nsample2.cols} is the number
			of samples in the disease group, \code{dcoex.genes} contains the genes
			contributing to the patterns and \code{startfile} the corresponding starting
			points.
}

\references{ Kostka D, Spang R (2004): Finding disease specific alterations in the coexpression of genes  }

\author{ Dennis Kostka and Dominic Fabian }

\seealso{  }

\examples{

## generate somedata
m1<-dcoex.gendat(50,15)
tmp <- matrix(rnorm(450*15),ncol=15)
m1 = rbind(m1,tmp)
rm(tmp)
m2<-matrix(rnorm(7500), nrow=500,ncol=15)

s = sample(500)
m1 = m1[s,]
m2 = m2[s,]


## run dcoex with the data

testdata<-dcoex(m1,m2)

# list the located genes
testdata$dcoex.genes

# list the number of samples of sample1
testdata$nsample1.cols

# list the used binary vectores
testdata$startfile

############################################################

## if you want to use a startmatrix:
## e.g. testpattern:

testpattern=as.matrix(c(rep(1,50),rep(0,450)))

testdata3<-dcoex(m1,m2,testpattern)
   
}
\keyword{ misc }% at least one, from doc/KEYWORDS
