#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <R.h>
#include <Rdefines.h>
/*#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_permute_vector_double.h>
#include <gsl/gsl_statistics_double.h>*/
#include "glob.h"

extern gsl_matrix *exprs1;
extern gsl_matrix *exprs2;
extern gsl_matrix *smat;     /*-- startingpoint matrix             */

gsl_matrix *makegslmatrix(SEXP mat1, SEXP mat1row, SEXP mat1column)
{
   int matrix_row, matrix_col, i, j, count;
   double *pointerm1;
   gsl_matrix * matrix1;

   PROTECT(mat1       = AS_NUMERIC(mat1));
   PROTECT(mat1row    = AS_INTEGER(mat1row));
   PROTECT(mat1column = AS_INTEGER(mat1column));

   pointerm1        = NUMERIC_POINTER(mat1);
   matrix_row       = *INTEGER_POINTER(mat1row);
   matrix_col       = *INTEGER_POINTER(mat1column);

   matrix1=gsl_matrix_alloc(matrix_row, matrix_col);
   count=0;
   for(i=0; i < matrix_row; i++)
   {
      for(j=0; j<matrix_col; j++)
      {
         gsl_matrix_set(matrix1,i,j,*(pointerm1+(matrix_row*j)+i ));/* + j hoch i  */
      }
          count++;
   }

   UNPROTECT(3);

   return(matrix1);
}

gsl_vector* makegslvector(SEXP vector, SEXP vec_rows)
{
   int vector_rows,i;
   double *pointvec;
   gsl_vector * R_vector;

   PROTECT(vector     = AS_NUMERIC(vector));
   PROTECT(vec_rows   = AS_INTEGER(vec_rows));

   pointvec       =  NUMERIC_POINTER(vector);
   vector_rows    = *INTEGER_POINTER(vec_rows);

   R_vector=gsl_vector_alloc(vector_rows);

   for(i=0; i < (vector_rows); i++)
   {
      gsl_vector_set(R_vector,i,*(pointvec+i));
   }

   UNPROTECT(2);
   return(R_vector);

}


SEXP
dcoex(  SEXP exprs1_inp,
        SEXP exprs2_inp,
        SEXP rows_inp,
        SEXP exprs1_cols_inp,
        SEXP exprs2_cols_inp,
        SEXP check_sfile_inp,
        SEXP Start_inp,
        SEXP Start_col_inp,
	SEXP alpha_inp)

{

   int rows,
       exprs1_cols,
       exprs2_cols,
       i,
       check_sfile,
       Start_cols,
       count,       /*-- count the number of passes     */
       run,         /*-- number of passes / columns     */
       have_sfile,  /*-- have startfile?                */
       conv,        /*-- convergence or maxit           */
       S_runs,
       countrows;

   double alpha,       /*-- tuning parameter               */
          chance,
          r;           /*-- final residual                 */


   gsl_matrix *smat;
   gsl_vector *track;
   SEXP output;


   PROTECT(rows_inp   = AS_INTEGER(rows_inp));
   PROTECT(exprs1_cols_inp   = AS_INTEGER(exprs1_cols_inp));
   PROTECT(exprs2_cols_inp   = AS_INTEGER(exprs2_cols_inp));
   PROTECT(check_sfile_inp   = AS_INTEGER(check_sfile_inp));
   PROTECT(alpha_inp = AS_NUMERIC(alpha_inp));
   PROTECT(Start_inp   = AS_INTEGER(Start_inp));

   rows = *INTEGER_POINTER(rows_inp);
   exprs1_cols = *INTEGER_POINTER(exprs1_cols_inp);
   exprs2_cols = *INTEGER_POINTER(exprs2_cols_inp);
   check_sfile = *INTEGER_POINTER(check_sfile_inp);
   alpha = *NUMERIC_POINTER(alpha_inp);

   gsl_rng_env_setup();

   srand((unsigned)time(NULL));
   gsl_rng_default_seed=rand();

   /* some output for the console... */

   printf("\nresults of dcoex:\n");

   exprs1=makegslmatrix (exprs1_inp, rows_inp, exprs1_cols_inp);

   exprs2=makegslmatrix (exprs2_inp, rows_inp, exprs2_cols_inp);

/*   alpha = 0.5;   */  /* default = 0.5 */
   chance = 0.3;
   i = 0;
   count = 0;
   countrows = 0;

   if (check_sfile == 1)
   {
      Start_col_inp   = AS_INTEGER (Start_col_inp);
      Start_cols      = *INTEGER_POINTER (Start_col_inp);
      output          = allocMatrix (REALSXP, (rows+1), Start_cols);
      track           = gsl_vector_alloc (rows);
      smat            = makegslmatrix (Start_inp, rows_inp, Start_col_inp);

      count=0;

      run=Start_cols;
      have_sfile=0;

      while (count < run )
      {
         count++;
         gsl_matrix_get_col(track,smat,(count-1));

         r=0;  /* set final_residual to zero */

         printf("\ncolumn %i:",count);

         if (gsl_vector_isnull(track)==1)
         {
            gsl_matrix_set_col(smat,(count-1),track);
            printf("\nis a zero column");
         }
         else
         {
            fpattern(smat,&r,&conv, alpha,track,have_sfile,count,chance);
         }

         countrows=(rows)*(count-1);

         /* Set the last value of output to the finalresidual */
         REAL (output)[(countrows + rows) + (count-1)] = r;

         /* fill the rest of output with smat */
         for (i=0; i<rows; i++)
         {
            REAL (output)[i+(countrows)+(count-1)]=gsl_matrix_get(smat,i,(count-1));
         }

      }
   }

   if (check_sfile == 2)
   {
      S_runs = *INTEGER_POINTER(Start_inp);

      output          = allocMatrix (REALSXP, (rows+1), S_runs);
      track           = gsl_vector_alloc (rows);
      smat = gsl_matrix_alloc(rows,S_runs);

      count=0;

      run=S_runs;
      have_sfile=-1;

      while (count < run )
      {
         count++;
         gsl_matrix_get_col(track, smat, (count-1));

         r=0;  /* set final_residual to zero */

         printf("\ncolumn %i:",count);

/*         if (gsl_vector_isnull(track)==1)
         {
            gsl_matrix_set_col(smat,(count-1),track);

            printf("\nis a zero column");
         }

         else
*/         {
            fpattern(smat,&r,&conv, alpha,track,have_sfile,count,chance);
         }

         countrows=(rows)*(count-1);

         /* Set the last value of output to the finalresidual */
         REAL (output)[(countrows + rows) + (count-1)] = r;

         /* fill the rest of output with smat */
         for (i=0; i<rows; i++)
         {
            REAL (output)[i+(countrows)+(count-1)]=gsl_matrix_get(smat,i,(count-1));
         }

      }
   }


   else
   {

      output = allocVector(REALSXP, (rows+1));

      track=gsl_vector_alloc(rows);

      smat=gsl_matrix_alloc(rows,1);

      for (i=0; i<rows; i++)
          gsl_matrix_set(smat, i, 0, gsl_vector_get(track,i));

      /*gsl_rng_default_seed = rand();*/  /* could changed to every positive int */
/*      alpha                = 0.5;   */  /* default = 0.5 */
      chance= 0.3;
      run=1;
      i=0;
      have_sfile=-1;
      count=1;

      /*--- call actual function */

      fpattern(smat,&r,&conv, alpha,track,have_sfile,count,chance);

      /* Set the last value of output to the finalresidual */
      REAL (output)[(rows)] = r;

      for (i=0; i<rows; i++)
      {
         REAL (output)[i]=gsl_matrix_get(smat,i,0);
      }
   }

   gsl_matrix_free(smat);
   gsl_vector_free(track);
   gsl_matrix_free(exprs1);
   gsl_matrix_free(exprs2);

   UNPROTECT(6);
   return (output);
}
