/*---
progaram to find differentially co-regulated groups of
genes. */


/*  dcoexmain.c is part of dcoex.

    dcoex is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dcoex is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dcoex; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/*---
  program to find differentially coexpressed groups of
  genes. */

/* some comments...
-- argv[3] = 0 is an illegal instruction
-- read files or x runs with random vector; x = argv[3]
*/

#include "glob.h"

/*--- GLOBALS                                               */
gsl_matrix *exprs1;   /*-- expression values of group 1     */
gsl_matrix *exprs2;   /*-- expression values of group 2     */
gsl_matrix *smat;     /*-- startingpoint matrix             */
gsl_vector *rmeans1;  /*-- row-means of group 1             */
gsl_vector *rmeans2;  /*-- row-means of group 2             */
gsl_rng    *r_perm;   /*-- gsl random numbers               */


/*--- global scratch vectors                                */
gsl_vector *rind_c;      /*-- copy of row indices           */
gsl_vector *cbase1_c;    /*-- copy of column bases group 1  */
gsl_vector *cbase2_c;    /*-- copy of column bases group 2  */
gsl_matrix *scratch1;    /*-- copy of group 1               */
gsl_matrix *scratch2;    /*-- copy of group 2               */


/* ------------------------------------------------------------------
FPATTERN
--------------------------------------------------------------------*/

int
fpattern(gsl_matrix * imat,    /*-- to hold final indices            */
	 double *res,          /*-- to hold final score              */
	 int *conv,            /*-- bool: converged (==1) or maxit?  */
	 double alpha,         /*-- tuning parameter                 */
	 gsl_vector *track,    /*-- to hold track of scores          */
	 int have_sfile,       /*-- have startfile?                  */
	 int count,             /*-- col of the startmatrix           */
	 double chance
	 )
{

  int  nr1,       /*-- number of rows in group 1                  */
       nr2,       /*-- number of rows in group 2                  */
       nc1,       /*-- number of cols in group 1                  */
       nc2,       /*-- number of cols in group 2                  */
       nr3, nc3,  /*-- number od cols/rows of Startingpoint data  */
       i,         /*-- index var                                  */
       j,         /*-- index var                                  */
       nbc,       /*-- number of patterns --> fixme               */
       l1,        /*-- number of pos. actions --> fixme           */
       lastind;   /*-- index of the gene for last action          */


  double  rprob;  /*-- prob. for a gene to be included in random starting pt.    */

  gsl_vector_view  col;      /*-- to copy indices from pattern do index matrix   */

  gsl_vector      *ones1,    /*-- used to calculate row means                    */
		  *ones2;    /*-- used to calculate row means                    */
		    /*-- holds the residuals while sampling             */

  gsl_matrix      *orig1,    /*-- to hold unscaled data                          */
		  *orig2;    /*-- to hold unscaled data                          */

  gsl_rng         *rgen;     /*-- gsl random numbers                             */

  Pattern         *bc;       /*-- the pattern currently optimized                */

  Action          *ac;       /*-- actions to explor neighborhood                 */

  const gsl_rng_type * T;


  /*--- the globals need to exist! */
  nr1 = exprs1->size1;
  nr2 = exprs2->size1;

  if (have_sfile !=-1)
    {
      nr3 = imat  ->size1;
      nc3 = imat  ->size2;
    }
  else  nr3=1; /*need for ptrn_rind in case of no starting point*/
  nc1 = exprs1->size2;
  nc2 = exprs2->size2;


  /*--- save unscaled data */
  orig1 = gsl_matrix_alloc(nr1,nc1);
  orig2 = gsl_matrix_alloc(nr2,nc2);

  gsl_matrix_memcpy(orig1, exprs1);
  gsl_matrix_memcpy(orig2, exprs2);

  /*--- set some pars */
  nbc   = 1;
  rprob = 1;  /*-- for random starting set */


 /*--- random numbers  */
  T      = gsl_rng_default;
  r_perm = gsl_rng_alloc (T);
  rgen   = r_perm;

  /*--- scale data */
  matrix_scale(exprs1);
  matrix_scale(exprs2);

  /*--- calculare row means  */
  rmeans1 = gsl_vector_alloc(nr1);
  rmeans2 = gsl_vector_alloc(nr2);
  ones1   = gsl_vector_alloc(nc1);
  ones2   = gsl_vector_alloc(nc2);

  gsl_vector_set_all(ones1, 1.);
  gsl_vector_set_all(ones2, 1.);

  gsl_blas_dgemv(CblasNoTrans, 1., exprs1, ones1, 0., rmeans1);
  gsl_blas_dgemv(CblasNoTrans, 1., exprs2, ones2, 0., rmeans2);

  gsl_vector_scale(rmeans1, 1/(double)nc1);
  gsl_vector_scale(rmeans2, 1/(double)nc2);

  /*--- allocate and initialize vector of patterns */
  bc = ptrn_alloc(nbc);


  /*--- allocate vector of row actions */
  ac = (Action *)malloc(nr1*sizeof(Action)); /* struct contains only built in types */

  /*--- allocate global scratch vars*/
  rind_c   = gsl_vector_alloc(nr1);
  cbase1_c = gsl_vector_alloc(nc1);
  cbase2_c = gsl_vector_alloc(nc2);
  scratch1 = gsl_matrix_alloc(nr1,nc1);
  scratch2 = gsl_matrix_alloc(nr2,nc2);

   /*-----------------------------------------------
    START ALGO
    ------------------------------------------------*/

  *conv=0;

      /*--- initialize indices --> random starting point --> fixme */


      ptrn_rind(bc,count,rprob, nr3, imat, have_sfile,chance);

      /*--- calculate all values (other than indices) */
      ptrn_val(bc);
      lastind=-1;

      /*------------------------------------------------------------------------*/
      for(j=0; j<nr1; j++) /* nr1=nr2 | max old: 20000*/
        {

	    /*--- calculate gains */
	    for(i=0; i<nr1; i++)
	      {
		 ac_val(ac+i, bc,i,alpha);
	      }

	     /*--- sort actions */
             ac_sort(ac,nr1); /* sort actions */

	     /*--- break if the largest gain is <= zero */
	     if((ac+(nr1-1))->gain <=0)
		{
		   *conv = 1;
		   break;
		}

	      /*--- count positive actions */
	      l1 = ac_count(ac, nr1);

/*	      printf("number of positive actions (%d)\t step number (%d) residuals (%f)\n",l1,j,(bc->res));*/

	       /*--- shuffle actions */
	       ac_rwo(ac+(nr1-l1-1),l1,l1);

	       /*--- figure out the top 1% of positive actions */
	       l1 = ((int) l1/100.) +1;

	       /*--- perform them (after shuffeling) */
	       ac_perf(ac+(nr1-l1),l1);

	       /*--- check if we are 'sufficiently' small */
	       if((bc)->res < 0.00001)
		 {
		    *conv = 1;
		    break;
		 }

	       /*--- report progress */
/*               if((j+1) % 10 == 0)
                  {

		      printf("%e  (%d) \t %d \n", (bc->res),j,i);
		  }*/

	       /*--- check for oscillations */
	       if(l1==1)
	         {
		    if((ac+(nr1-l1))->ind == lastind)
		      {
			 break;
			 fprintf(stderr,"\nOSCILLATING\n");
		      }
	            lastind = (ac+(nr1-l1))->ind;
	          }

	       /*--- save to track */
	       gsl_vector_set(track,j,bc->res);  /* keep track of reses */
	}


/*------------------------------------------------------------------------*/

     if(j==nr1)
        {
           *conv = -1;
        }

          /*--- write final pattern row indices in matrix */
           col  = gsl_matrix_column(imat, (count-1));           /* --> fixme         */




           gsl_vector_memcpy(&col.vector, (bc)->rind);  /* copy index vector */
           *res = bc->res;                              /* final residual    */
           printf("\n final residual : -- %e --\n", bc->res);


  /*-- clean up memory */

  free            (ac      );
  gsl_matrix_free (orig1   );
  gsl_matrix_free (orig2   );
  ptrn_free       (bc,nbc  );
  gsl_matrix_free (scratch1);
  gsl_matrix_free (scratch2);
  gsl_vector_free (ones1   );
  gsl_vector_free (ones2   );
  gsl_rng_free    (r_perm  );
  gsl_vector_free (rmeans1 );
  gsl_vector_free (rmeans2 );
  gsl_vector_free (cbase1_c);
  gsl_vector_free (cbase2_c);
  gsl_vector_free (rind_c  );

  return 0;
}
